@@ -1,7 +1,7 @@
 r"""UUID draft version objects (universally unique identifiers).
-This module provides the functions uuid6() and uuid7() for
-generating version 6 and 7 UUIDs as specified in
-https://github.com/uuid6/uuid6-ietf-draft.
+This module provides the functions uuid6(), uuid7(), and uuid8() for
+generating version 6, 7, and 8 UUIDs as specified in
+https://github.com/ietf-wg-uuidrev/rfc4122bis
 """
 
 import secrets
@@ -39,7 +39,7 @@ class UUID(uuid.UUID):
         if not 0 <= int < 1 << 128:
             raise ValueError("int is out of range (need a 128-bit value)")
         if version is not None:
-            if not 6 <= version <= 7:
+            if not 6 <= version <= 8:
                 raise ValueError("illegal version number")
             # Set the variant to RFC 4122.
             int &= ~(0xC000 << 48)
@@ -62,6 +62,8 @@ class UUID(uuid.UUID):
                 | (self.time_hi_version & 0x0FFF)
             )
         if self.version == 7:
+            return self.int >> 80
+        if self.version == 8:
             return (self.int >> 80) * 10**6 + _subsec_decode(self.subsec)
         return super().time
 
@@ -76,10 +78,15 @@ def _subsec_encode(value: int) -> int:
 
 _last_v6_timestamp = None
 _last_v7_timestamp = None
+_last_v8_timestamp = None
 
 
 def uuid6(clock_seq: int = None) -> UUID:
-    r"""Generate a UUID from sequence number, and the current time.
+    r"""UUID version 6 is a field-compatible version of UUIDv1, reordered for
+    improved DB locality. It is expected that UUIDv6 will primarily be
+    used in contexts where there are existing v1 UUIDs. Systems that do
+    not involve legacy UUIDv1 SHOULD consider using UUIDv7 instead.
+
     If 'clock_seq' is given, it is used as the sequence number;
     otherwise a random 14-bit sequence number is chosen."""
 
@@ -94,38 +101,55 @@ def uuid6(clock_seq: int = None) -> UUID:
     _last_v6_timestamp = timestamp
     if clock_seq is None:
         clock_seq = secrets.randbits(14)  # instead of stable storage
-    node = secrets.randbits(48)
     time_high_and_time_mid = (timestamp >> 12) & 0xFFFFFFFFFFFF
     time_low_and_version = timestamp & 0x0FFF
     uuid_int = time_high_and_time_mid << 80
     uuid_int |= time_low_and_version << 64
     uuid_int |= (clock_seq & 0x3FFF) << 48
-    uuid_int |= node
+    uuid_int |= secrets.randbits(48)
     return UUID(int=uuid_int, version=6)
 
 
 def uuid7() -> UUID:
-    r"""The UUIDv7 format is designed to encode a Unix timestamp with
-    arbitrary sub-second precision.  The key property provided by UUIDv7
-    is that timestamp values generated by one system and parsed by
-    another are guaranteed to have sub-second precision of either the
-    generator or the parser, whichever is less.  Additionally, the system
-    parsing the UUIDv7 value does not need to know which precision was
-    used during encoding in order to function correctly."""
+    r"""UUID version 7 features a time-ordered value field derived from the
+    widely implemented and well known Unix Epoch timestamp source, the
+    number of milliseconds seconds since midnight 1 Jan 1970 UTC, leap
+    seconds excluded. As well as improved entropy characteristics over
+    versions 1 or 6.
+
+    Implementations SHOULD utilize UUID version 7 over UUID version 1 and
+    6 if possible."""
 
     global _last_v7_timestamp
 
     nanoseconds = time.time_ns()
-    if _last_v7_timestamp is not None and nanoseconds <= _last_v7_timestamp:
-        nanoseconds = _last_v7_timestamp + 1
-    _last_v7_timestamp = nanoseconds
+    timestamp_ms, _ = divmod(nanoseconds, 10**6)
+    if _last_v7_timestamp is not None and timestamp_ms <= _last_v7_timestamp:
+        timestamp_ms = _last_v7_timestamp + 1
+    _last_v7_timestamp = timestamp_ms
+    uuid_int = (timestamp_ms & 0xFFFFFFFFFFFF) << 80
+    uuid_int |= secrets.randbits(76)
+    return UUID(int=uuid_int, version=7)
+
+
+def uuid8() -> UUID:
+    r"""UUID version 8 features a time-ordered value field derived from the
+    widely implemented and well known Unix Epoch timestamp source, the
+    number of nanoseconds seconds since midnight 1 Jan 1970 UTC, leap
+    seconds excluded."""
+
+    global _last_v8_timestamp
+
+    nanoseconds = time.time_ns()
+    if _last_v8_timestamp is not None and nanoseconds <= _last_v8_timestamp:
+        nanoseconds = _last_v8_timestamp + 1
+    _last_v8_timestamp = nanoseconds
     timestamp_ms, timestamp_ns = divmod(nanoseconds, 10**6)
     subsec = _subsec_encode(timestamp_ns)
     subsec_a = subsec >> 8
     subsec_b = subsec & 0xFF
-    rand = secrets.randbits(54)
     uuid_int = (timestamp_ms & 0xFFFFFFFFFFFF) << 80
     uuid_int |= subsec_a << 64
     uuid_int |= subsec_b << 54
-    uuid_int |= rand
-    return UUID(int=uuid_int, version=7)
+    uuid_int |= secrets.randbits(54)
+    return UUID(int=uuid_int, version=8)
